/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "boardwidget.h"
#include "tilewidget.h"
#include <QtGui>

//Widget definition of the board module
BoardWidget::BoardWidget( QWidget *parent) : QWidget( parent) {
	//vertical layout
	QVBoxLayout *vbox_Layout = new QVBoxLayout( this );
	
	vbox_Layout->setSpacing(0);
	vbox_Layout->setMargin(0);
	
	QWidget *horLayout1 = new QWidget;
	QHBoxLayout *hboxLayout1 = new QHBoxLayout(horLayout1);
	hboxLayout1->setSpacing(0);
	hboxLayout1->setMargin(0);
	QWidget *horLayout2 = new QWidget;
	QHBoxLayout *hboxLayout2 = new QHBoxLayout(horLayout2);
	hboxLayout2->setSpacing(0);
	hboxLayout2->setMargin(0);
	
	//first line : target tiles
	//TileWidget 10, 11 et 12
	hboxLayout1->addStretch( 1 );
	
	for(int Wi=0; Wi<3; Wi++) {
		TileWidget *tile= new TileWidget(horLayout1, Wi+10, 0);
		hboxLayout1->addWidget( tile );
		connect( this, SIGNAL(refresh(int, int)), tile, SLOT(displayRefresh(int, int)) );
		connect( tile, SIGNAL(change(int, bool)), this, SLOT(targetTileClicked(int, bool)) );
		connect( this, SIGNAL(variableTile(bool)), tile, SLOT(isVariable(bool)) );
	}
	
	hboxLayout1->addStretch( 1 );
	
	//second line : selection tiles
	horLayout2->setFixedHeight( 43 );
	
	for(int Wi=0; Wi<6; Wi++) {
		TileWidget *tile= new TileWidget(horLayout2, Wi, 1);
		hboxLayout2->addWidget( tile );
		connect( this, SIGNAL(refresh(int, int)), tile, SLOT(displayRefresh(int, int)) );
		connect( this, SIGNAL(changeStatus(int, int)), tile, SLOT(updateStatus(int, int)) );
		connect( tile, SIGNAL(change(int, bool)), this, SLOT(selectionTileClicked(int, bool)) );
		connect( this, SIGNAL(variableTile(bool)), tile, SLOT(isVariable(bool)) );
		connect( tile, SIGNAL(value(int,int)), this, SIGNAL(tileClicked(int,int)) );
	}

	vbox_Layout->addWidget(horLayout1);
	vbox_Layout->addWidget(horLayout2);

	//init digitList
	digitList.clear();
	digitList.assign(3,0);
}

void BoardWidget::initTarget(int newTarget) {
	digitList.clear();
	digitList.assign(3,0);
	digitList[0]=newTarget/100;
	digitList[1]=(newTarget/10)%10;
	digitList[2]=newTarget%10;
}

void BoardWidget::initSelection(selection t_ini) {
	t=t_ini;
	tileList = t_ini.getTileList();
}

void BoardWidget::updateTargetDisplay() {
	for(int Wi=0; Wi<3; Wi++) {
		emit refresh(Wi+10, digitList[Wi]);
	}
}

void BoardWidget::updateTileDisplay() {
	int n = tileList.size();
	for(int Wi=0; Wi<6; Wi++) {
		//security on the size of tileList
		if(Wi<n) {
			emit refresh(Wi, tileList[Wi]);
		} else {
			emit refresh(Wi, 0);
		}
	}
}

void BoardWidget::updateTileNumber() {
	int n = tileList.size();
	if(selection::nb_tiles != n) {
		tileList.clear();
		tileList.assign(6,0);
	}
	updateTileDisplay();
}

//called everytime a target tile is clicked
void BoardWidget::targetTileClicked(int tile_id, bool increase) {
	if(increase) {
		digitList[tile_id-10]=digitList[tile_id-10]+1;
	} else {
		digitList[tile_id-10]=digitList[tile_id-10]-1;
	}
	
	int newTarget=digitList[0]*100+digitList[1]*10+digitList[2];
	
	if(newTarget>=selection::minimalTarget && newTarget <= selection::maximalTarget) {
		digitList[0]=newTarget/100;
		digitList[1]=(newTarget/10)%10;
		digitList[2]=(newTarget)%10;
		
		updateTargetDisplay();
		emit targetChanged(newTarget);
		return;
	}
	
	//cancel modification
	if(increase) {
		digitList[tile_id-10]=digitList[tile_id-10]-1;
	} else {
		digitList[tile_id-10]=digitList[tile_id-10]+1;
	}
}

//called everytime a selection tile changes
void BoardWidget::selectionTileClicked(int tile_id, bool increase) {
	//security against any possible programming errors
	int n = tileList.size();
	if(tile_id >= n || tile_id<0) {
		cout << "Invalid argument in BoardWidget::selectionTileClicked";
		return;
	}
	
	tile currentTile=tileList[tile_id];
	tile newTile=0;
	if(increase){
		newTile=leastTileSup(t,currentTile);
	} else{
		newTile=leastTileInf(t,currentTile);
	}
	
	if(newTile!=0){
		tileList[tile_id]=newTile;
		t.tab[currentTile]--;
		t.tab[newTile]++;
		
		//display
		updateTileDisplay();
		emit selectionChanged(t);
	}
}

void BoardWidget::randomSelection() {
	//new selection
	t.tab.clear();
	
	tileList.clear();
	tileList.assign(selection::nb_tiles,0);
	int i;
	int randomTile;
	
	vector<tile> possibleTiles;	//default : [1;1;2;2;...;75;100]
	map<tile, int>::iterator it;
	for(it=selection::ref.begin();it!=selection::ref.end();it++){
		for(i=1;i<=it->second;i++){
			possibleTiles.push_back(it->first);
		}
	}
	
	//TODO : be sure that nb_possibleTiles>=selection::nb_tiles
	int nb_possibleTiles=possibleTiles.size();	//default : 24
	
	for(i=0;i<selection::nb_tiles;i++){
		randomTile=rand()%(nb_possibleTiles-i);
		tileList[i]=possibleTiles[randomTile];
		possibleTiles.erase(possibleTiles.begin()+randomTile);
		t.tab[tileList[i]]++;
	}
}

void BoardWidget::assignTarget(int value_targetType) {
	digitList.clear();
	digitList.assign(3,0);
	digitList[0]=value_targetType/100;
	digitList[1]=(value_targetType/10)%10;
	digitList[2]=(value_targetType)%10;
	target = value_targetType;
}
