/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef CALCULATION_H
#define CALCULATION_H

#include <set>
#include <list>
#include <string>
#include <iostream>
#include <algorithm>
#include <sstream>

using namespace std;

typedef unsigned char tile;
typedef long targetType;  //warning : signed number are needed, because canonization functions use negative numbers

class selection;

class calcul {
  public:
	// DATA
	bool isTile;
	tile pl;
	
	bool isAdditive;
	list<calcul> sub_calculs;	//list of sub_calculs of depth 1
	bool sign;			//sign of the calcul in the parent calcul (true=positive, false=negative)
	targetType value;
	
	// DISPLAY
	static bool topLevel; 		//static variable to indicate if we are at the top level
	string display() const;
	
	// FUNCTIONS
	
	//initialization functions
	calcul();					//default initialization (additive calcul)
	calcul(const tile&); 		//initialization from a tile
	
	//operator == is used in the definition of operator <
	//operator < is necessary to sort a list of calculs with the stl sort function
	bool operator==(const calcul& a) const;
	bool operator<(const calcul& a) const;
	
	//change signs
	void reverseSign();
	
	//operations with 2 calculs
	calcul operator+(const calcul&) const;
	calcul operator-(const calcul&) const;
	calcul operator*(const calcul&) const;
	calcul operator/(const calcul&) const;
	calcul reunion(const calcul&, bool additiveType, bool opposition) const;
	
	//functions to analyze the calculs (used in the canonization)
	list<calcul> sub_calculs_list() const;
	list<targetType> children_reachableTarget(bool) const;
	list<targetType> subSet_reachableTarget(bool) const;
	list<targetType> subTargetList() const;
	selection tileList() const;
	selection valuesList() const;
	int length() const;		//used in game::displayBase2()
	
	//------canonization-------------
	bool isCanonical() const; //global function to canonize with the functions below
	static bool cano1_active, cano2_active, cano3_active;	//activate/desactivate a canonization
	bool isCanonical_1() const;
	bool isCanonical_2() const;
	bool isCanonical_3() const;
};

#endif
