/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "clockwidget.h"

#include <QtGui>

//50 seconds available by default
int ClockWidget::nb_sec=50;

ClockWidget::ClockWidget( QWidget *parent ) : QWidget( parent ) {
	subdivision = 50;	//number of subdivisions of the progress bar
	
	pbar = new QProgressBar(this);
	pbar->setRange(0, subdivision);
	pbar->setValue(0);
	pbar->setTextVisible(false);	//don't display progress as a percentage
	
	pbar->setPalette(orangePalette); //color the bar with the theme color
	
	QHBoxLayout *hbLayout = new QHBoxLayout(this);
	hbLayout->setSpacing(0);
	hbLayout->setMargin(0);
	hbLayout->addWidget( pbar );
	
	timer = new QTimer(this);
	connect(timer, SIGNAL(timeout()), this, SLOT(increaseTime()));
}

//launch timer
void ClockWidget::startClock() {
	timer->start(1000*nb_sec/subdivision);
	timeUnits=0;
	pbar->setValue(timeUnits);
}

//stop timer
void ClockWidget::stopClock() {
	timer->stop();
}

//init timer
void ClockWidget::initClock() {
	pbar->setValue(0);
}

//progress of one time unit
void ClockWidget::increaseTime() {
	timeUnits++;			//increment of one time unit
	pbar->setValue(timeUnits);
	
	//emit "timeElapsed" signal when there is no time left
	if (timeUnits==subdivision) {
		timer->stop();
		emit timeElapsed();
	}
}
