/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GAME_H
#define GAME_H

#include "calculation.h"
#include "selection.h"

#include <QString>

#include <qthread.h>
#include <qmutex.h>

#include <fstream>
#include <sstream>

typedef set<calcul> stock;

class game : public QThread {
	Q_OBJECT
	
  public :
	void run();	//to launch the computation as a separate thread
	
	// DATA
	selection givenSelection;
	vector<selection> subSelections;
	vector<stock> completeBase;
	
	targetType lim_inf_inf;
	targetType lim_inf_sup;
	targetType lim_sup_inf;
	targetType lim_sup_sup;
	
	targetType lim_inf;
	targetType lim_sup;
	
	//communication inside the class
	bool last_subSelection;	//indicate whether we are on the last subSelection or not
	int currentStock;			//stock currently used
	
	// FUNCTIONS
	void launchAnalysis();	//main function
	void init(const selection &t, const int &liminf, const int &limsup);	//initialization
	void combinations(const calcul& A, const calcul& B);			//compute all the possible combinations of A and B
	void addToBase(const calcul& A);			//add a calcul to the base (in the current stock)
	void deleteNonCano(); 						//delete non canonical positions from the current stock
	bool inTheRange(const calcul& C);			//find if a calcul has a value that we need and change the limits if necessary
	void deleteOutOfRange();					//delete from the base the positions "out of the range"
	
	QString displayBase() const;
	QString displayProblem() const;
	int distanceToSolution() const;
	vector<bool> reachedTarget() const;		//vector of all possible targets, 0 if not reached, 1 if reached
};

#endif
