/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwidget.h"
#include "selectionwidget.h"
#include <iostream>
#include <ctime>
#include <QtGui>

/*-------------------------------------------------------------------------------------------------
// Main Widget
//-----------------------------------------------------------------------------------------------*/

MainWidget::MainWidget( QWidget *parent ) : QMainWindow( parent ) {
	//icon on the upper left corner and in the taskbar
	setWindowIcon(QIcon(QString(":/images/kitsune-16x16.png") ));
	
	//displayed text definition
	//note : since the "MainWidget" class contains the Q_OBJECT macro, direct access to "tr" function is possible
	
	kitsuneVersionText="Kitsune "+ global::kitsuneVersion;
	foxTextDefault=tr("Kitsune");
	
	//mapper for reading text files
	readText_Mapper=new QSignalMapper(this);
	connect( readText_Mapper, SIGNAL(mapped(int)), this, SLOT(readText(int)) );
	
	//mapper for choosing the number of tiles
	tileNumber_Mapper=new QSignalMapper(this);
	connect( tileNumber_Mapper, SIGNAL(mapped(int)), this, SLOT(setTileNumber(int)) );
	
	//mapper for choosing the game mode
	mode_Mapper=new QSignalMapper(this);
	connect( mode_Mapper, SIGNAL(mapped(int)), this, SLOT(changeMode(int)) );
	
	//title bar
	setWindowTitle(QString(tr("Kitsune")));
	
	//menu bar
	QMenuBar *MenuBar = new QMenuBar( this );
	setMenuBar(MenuBar);
	MenuBar->setFont( font_normal );
	
	fileMenu = new QMenu ( this );
	QAction *optionsAction = new QAction(this);
	QAction *fileExitAction = new QAction(this);
	QAction *scoreDetailsAction = new QAction(this);
	optionsAction->setText(tr("&Options"));
	fileExitAction->setText(tr("E&xit"));
	scoreDetailsAction->setText(tr("&Score details"));
	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(optionsAction);
	fileMenu->addAction(scoreDetailsAction);
	fileMenu->addAction(fileExitAction);
	fileMenu->setFont( font_normal );
	
	tileMenu = new QMenu ( this );
	tileMenu = menuBar()->addMenu(tr("&Tiles"));
	tileMenu->setFont( font_normal );
	
	tileGroup = new QActionGroup(this);
	int i;
	for(i=3; i<=6; i++) {
		QAction *tileAction = new QAction( this );
		tileAction->setText(tr("%1 tiles").arg(i));
		tileAction->setCheckable(true);
		if(selection::nb_tiles==i) {
			tileAction->setChecked(true);
		}
		tileGroup->addAction(tileAction);
		tileMenu->addAction(tileAction);
		
		connect( tileAction, SIGNAL( triggered() ), tileNumber_Mapper, SLOT(map()) );
		tileNumber_Mapper->setMapping(tileAction, i);
	}
	
	tileMenu->addSeparator();
	QAction *selectTilesAction = new QAction( this );
	selectTilesAction->setText(tr("Authorized tiles"));
	tileMenu->addAction(selectTilesAction);
	connect( selectTilesAction, SIGNAL( triggered() ), this, SLOT(selectTiles()) );
	
	modeMenu = new QMenu ( this );
	tvAction = new QAction( this );
	challengeAction = new QAction( this );
	trainingAction = new QAction( this );
	tvAction->setText(tr("TV Mode"));
	tvAction->setCheckable(true);
	challengeAction->setText(tr("Challenge Mode"));
	challengeAction->setCheckable(true);
	trainingAction->setText(tr("Training Mode"));
	trainingAction->setCheckable(true);
	
	QActionGroup *modeGroup = new QActionGroup(this);
	modeGroup->addAction(tvAction);
	modeGroup->addAction(challengeAction);
	modeGroup->addAction(trainingAction);
	challengeAction->setChecked(true);
	
	modeMenu = menuBar()->addMenu(tr("&Game mode"));
	modeMenu->addAction(tvAction);
	modeMenu->addAction(challengeAction);
	modeMenu->addAction(trainingAction);
	modeMenu->setFont( font_normal );
	
	infoMenu = new QMenu ( this );
	QAction *helpAideAction = new QAction( this );
	QAction *helpLicenceAction = new QAction( this );
	QAction *helpAboutAction = new QAction( this );
	helpAideAction->setText(tr("&Help"));
	helpLicenceAction->setText(tr("&Licence"));
	helpAboutAction->setText(tr("&About"));
	infoMenu = menuBar()->addMenu(tr("&Information"));
	infoMenu->setFont( font_normal );
	infoMenu->addAction(helpAideAction);
	infoMenu->addAction(helpAboutAction);
	infoMenu->addAction(helpLicenceAction);
	
	//menu bar signal<--->slot connections
	connect( fileExitAction, SIGNAL( triggered() ), this, SLOT( close() ) );
	connect( optionsAction, SIGNAL( triggered() ), this, SLOT( options() ) );
	connect( helpAideAction, SIGNAL( triggered() ), readText_Mapper, SLOT(map()) );
	connect( helpLicenceAction, SIGNAL( triggered() ), readText_Mapper, SLOT(map()) );
	connect( helpAboutAction, SIGNAL( triggered() ), readText_Mapper, SLOT(map()) );
	readText_Mapper->setMapping(helpAideAction, 2);
	readText_Mapper->setMapping(helpLicenceAction, 1);
	readText_Mapper->setMapping(helpAboutAction, 0);
	
	connect( tvAction, SIGNAL( triggered() ), mode_Mapper, SLOT(map()) );
	connect( challengeAction, SIGNAL( triggered() ), mode_Mapper, SLOT(map()) );
	connect( trainingAction, SIGNAL( triggered() ), mode_Mapper, SLOT(map()) );
	mode_Mapper->setMapping(tvAction, global::ModeTV);
	mode_Mapper->setMapping(challengeAction, global::ModeRandom);
	mode_Mapper->setMapping(trainingAction, global::ModeTraining);
	
	//horizontal parent widget
	QWidget *hbox = new QWidget( this );
	QHBoxLayout *hboxLayout = new QHBoxLayout( hbox );
	
	hboxLayout->setSpacing(1);
	hboxLayout->setMargin(5);
	
	setCentralWidget(hbox);
	
	//vertical left box
	QWidget *vboxA = new QWidget( hbox );
	QVBoxLayout *vboxLayoutA = new QVBoxLayout( vboxA );
	
	vboxA->setFixedWidth(292);
	vboxLayoutA->setSpacing(0);
	vboxLayoutA->setMargin(3);
	
	//vertical middle box
	QWidget *vboxB = new QWidget( hbox );
	QVBoxLayout *vboxLayoutB = new QVBoxLayout( vboxB );
	
	vboxLayoutB->setSpacing(0);
	vboxLayoutB->setMargin(0);
	vboxB->setFixedWidth(200);
	
	//text on the right to display results
	//Warning : there is a bug in QTextBrowser that prevents to use setPlainText and setHtml together
	// (bug : If we click on a link in QTextBrowser, setPlainText displays text as link)
	//As a solution, we use a QTexEdit and a QTextBrowser, displayed at the same position
	//the QTextEdit is used everywhere, except when we display the message "Training mode"
	//The message "training mode" contains a link to kitsune web site, and this is why we need QTextBrowser
	QWidget *vboxC = new QWidget( hbox );
	QVBoxLayout *vboxLayoutC = new QVBoxLayout( vboxC );
	vboxLayoutC->setSpacing(0);
	vboxLayoutC->setMargin(0);
	
	textEditC = new QTextEdit( vboxC );
	textEditC->setReadOnly(true);
	textEditC->setFixedWidth(292);
	textEditC->setFont( font_monospace );
	textEditC->setFocusPolicy(Qt::NoFocus);
	
	textEdit2 = new QTextBrowser( vboxC );
	textEdit2->setReadOnly(true);
	textEdit2->setOpenExternalLinks(true);
	textEdit2->setFixedWidth(292);
	textEdit2->setFont( font_monospace );
	textEdit2->setFocusPolicy(Qt::NoFocus);
	textEdit2->setVisible(false);
	
	vboxLayoutC->addWidget(textEditC);
	vboxLayoutC->addWidget(textEdit2);
	vboxC->setFixedWidth(vboxC->sizeHint().width());
	
	//insert widget in the layout
	hboxLayout->addStretch( 1 );
	hboxLayout->addWidget( vboxA );
	hboxLayout->addStretch( 1 );
	hboxLayout->addWidget( vboxB );
	hboxLayout->addStretch( 1 );
	hboxLayout->addWidget( vboxC );
	hboxLayout->addStretch( 1 );
	
	//----------left column--------------------------------------------------------------
	board = new BoardWidget( vboxA );
	
	QWidget *horLayoutw_solution = new QWidget( vboxA );
	QHBoxLayout *hboxLayoutw_solution = new QHBoxLayout(horLayoutw_solution);
	hboxLayoutw_solution->setSpacing(0);
	hboxLayoutw_solution->setMargin(0);
	w_solution= new SolutionWidget(horLayoutw_solution);
	
	connect( board, SIGNAL(targetChanged(int)), this, SLOT(targetChanged(int)) );
	connect( board, SIGNAL(selectionChanged(selection)), this, SLOT(selectionChanged(selection)) );
	connect( this, SIGNAL(variableTile(bool)), board, SIGNAL(variableTile(bool)) );
	connect( this, SIGNAL(changeStatus(int, int)), board, SIGNAL(changeStatus(int, int)) );

	//connections between board and w_solutionution module
	connect( w_solution, SIGNAL(changeStatus(int, int)), board, SIGNAL(changeStatus(int, int)) );
	connect( board, SIGNAL(tileClicked(int,int)), w_solution, SLOT(tileClicked(int,int)) );

	//progress bar for the remaining time
	clock=new ClockWidget( vboxA );
	clock->setFixedHeight( 15 );
	
	//insert widgets in the layout
	vboxLayoutA->addStretch( 1 );
	vboxLayoutA->addWidget( board );
	vboxLayoutA->addWidget( horLayoutw_solution );
	vboxLayoutA->addSpacing( 5 );
	vboxLayoutA->addWidget( clock );
	vboxLayoutA->addStretch( 1 );
	
	//third line : solving widget
	hboxLayoutw_solution->addStretch( 1 );
	hboxLayoutw_solution->addWidget( w_solution );
	hboxLayoutw_solution->addStretch( 1 );
	
	connect( w_solution, SIGNAL(victory()), this, SLOT(victory()) );
	connect( clock, SIGNAL(timeElapsed()), this, SLOT(defeat()) );
	
	//---------- middle column ---------------------------------------------------------------
	
	//win/loss numbers
	w_player = new PlayerWidget( vboxB );
	w_player->readWinLoss();
	connect( scoreDetailsAction, SIGNAL( triggered() ), w_player, SLOT(displayScoreDialog()) );
	
	//fox image
	QWidget *horLayoutB1 = new QWidget( vboxB );
	QHBoxLayout *hboxLayoutB1 = new QHBoxLayout(horLayoutB1);
	hboxLayoutB1->setSpacing(0);
	hboxLayoutB1->setMargin(0);
	
	horLayoutB1->setFixedHeight(150);
	
	pixmapLabelFox = new QLabel( horLayoutB1 );
	normalFox = new QPixmap(":/images/normal-150x150.png");
	happyFox = new QPixmap(":/images/happy-150x150.png");
	sadFox = new QPixmap(":/images/sad-150x150.png");
	pixmapLabelFox->setFixedSize( 150, 150 );
	
	hboxLayoutB1->addStretch( 1 );
	hboxLayoutB1->addWidget(pixmapLabelFox);
	hboxLayoutB1->addStretch( 1 );
	
	//text under the fox
	foxText = new QLabel( vboxB );
	foxText->setFont( font_big );
	foxText->setAlignment( Qt::AlignHCenter );
	
	//buttons
	pbplay = new QPushButton(tr("Play"), vboxB );
	pbRes = new QPushButton(tr("Solve"), vboxB );
	
	pbplay->setFont( font_normal );
	pbRes->setFont( font_normal );
	pbplay->setFixedWidth( 100 );
	pbRes->setFixedWidth( 100 );
	pbplay->setFocusPolicy(Qt::NoFocus);
	pbRes->setFocusPolicy(Qt::NoFocus);
	
	//mode image (tv, dice, file)
	QWidget *horLayoutB2 = new QWidget( vboxB );
	QHBoxLayout *hboxLayoutB2 = new QHBoxLayout(horLayoutB2);
	hboxLayoutB2->setSpacing(0);
	hboxLayoutB2->setMargin(0);
	
	horLayoutB2->setFixedHeight(48);
	
	television = new QPixmap(":/images/tv.png");
	dice = new QPixmap(":/images/random.png");
	tick = new QPixmap(":/images/problems.png");
	
	pbMode = new QPushButton("", vboxB );
	pbMode->setFlat( true );
	pbMode->setFixedSize( 48, 48 );
	pbMode->setFocusPolicy(Qt::NoFocus);
	pbMode->setIcon( *dice );
	pbMode->setIconSize( QSize(48,48) );
	connect( pbMode, SIGNAL(clicked()), this, SLOT(rotateMode()) );
	
	hboxLayoutB2->addStretch( 1 );
	hboxLayoutB2->addWidget(pbMode);
	hboxLayoutB2->addStretch( 1 );
	
	//insert widgets in the layout
	vboxLayoutB->addStretch( 3 );
	vboxLayoutB->addWidget( horLayoutB1 );
	vboxLayoutB->addStretch( 1 );
	vboxLayoutB->addWidget( foxText );
	vboxLayoutB->addStretch( 3 );
	vboxLayoutB->addWidget( pbplay );
	vboxLayoutB->addStretch( 1 );
	vboxLayoutB->addWidget( pbRes );
	vboxLayoutB->addStretch( 3 );
	vboxLayoutB->addWidget( w_player );
	vboxLayoutB->addStretch( 3 );
	vboxLayoutB->addWidget( horLayoutB2 );
	vboxLayoutB->addStretch( 3 );
	
	vboxLayoutB->setAlignment(pbplay,Qt::AlignHCenter);
	vboxLayoutB->setAlignment(pbRes,Qt::AlignHCenter);
	
	//----------Initializations----------------------------------------------------------------
	
	//true random values
	srand (time (NULL));
	
	//lock/unlock solving module
	connect( this, SIGNAL(lockModule(bool)), w_solution, SLOT(lockModule(bool)) );
	
	//click on the buttons
	connect( pbplay, SIGNAL(clicked()), this, SLOT(play()) );
	connect( pbRes, SIGNAL(clicked()), this, SLOT(solve()) );
	
	//init and connect class game (the class that performs the computation of the solutions)
	J =new game;
	connect( J, SIGNAL(finished()), this, SLOT(computationFinished()) );
	
	//parameters initialization
	initialize();
	
	//mode initialization
	problemStart=true; //note : this variable must be initialized BEFORE any call to changeMode
	global::currentMode = global::ModeTraining;
	changeMode(global::ModeRandom);
	targetCompute = false;
}

/*-------------------------------------------------------------------------------------------------
// Functions called by the menu bar
//-----------------------------------------------------------------------------------------------*/

//options
void MainWidget::options() {
	int oldMinimalTarget=selection::minimalTarget;
	int oldMaximalTarget=selection::maximalTarget;
	
	OptionsWidget opt( this );
	opt.exec();
	
	if(oldMinimalTarget!=selection::minimalTarget || oldMaximalTarget!=selection::maximalTarget){
		initialize();
	}
}

// "Info" menu
void MainWidget::readText(int num) {
	if(num==0) {
		//About
		PopupWidget popup( this, false, QString(":/txt/%1/about.html").arg(OptionsWidget::language), QString(kitsuneVersionText));
		popup.exec();
	} else if(num==1) {
		//licence
		PopupWidget popup( this, true, QString(":/txt/%1/licence.html").arg(OptionsWidget::language), QString(tr("License of Kitsune")));
		popup.exec();
	} else if(num==2) {
		//help
		PopupWidget popup( this, false, QString(":/txt/%1/help.html").arg(OptionsWidget::language), QString(tr("Help")));
		popup.exec();
	}
}

/*-------------------------------------------------------------------------------------------------
// Functions called when a button is clicked
//-----------------------------------------------------------------------------------------------*/

//start (action depends on the game mode, so we call a different function for each mode)
void MainWidget::play() {
	if(global::currentMode==global::ModeTV) {
		modeTV_game();
	} else
	if(global::currentMode==global::ModeRandom) {
		modeRandom_game();
	} else
	if(global::currentMode==global::ModeTraining) {
		modeTraining_game();
	}
}

void MainWidget::beginGame() {
	gameRunning=true;
	tooLate=false;
	reached =false;
	textEditC->setPlainText("");		//delete the "result" text on the right
	pbplay->setEnabled( false );
	pbRes->setEnabled(true);
	emit variableTile( false );
	
	//lock menus
	fileMenu->setEnabled(false);
	tileMenu->setEnabled(false);
	modeMenu->setEnabled(false);
	pbMode->setEnabled(false);
	
	w_solution->init();
	reinit_SelectionTiles( false );
	reinit_status();
	clock->initClock();
}

void MainWidget::modeTV_game() {
	//init interface, timer and variables
	beginGame();
	
	//launch computation in a separate thread
	launchComputation();
}

void MainWidget::modeTraining_game() {
	if(problemStart) {
		//select a problem file
		problemFileOpen();
		if(problems.problemList.size()>0) {
			problemIndex=-1;
			problemStart = false;
			pbplay->setText(tr("Next"));
			
			//we were displaying the "training mode" message with the browser
			//we need to mask it, and shows the usual textedit
			textEditC->setVisible(true);
			textEdit2->setVisible(false);
			
			//display in the textedit the introduction page of the selected problems file
			textEditC->setPlainText(problems.intro());
		}
	} else {
		saveResult();
		nextProblemInit();
	}
}

//save information about the result achieved by the player
void MainWidget::saveResult() {
	if(problemIndex>=0) {	
		problems.problemList[problemIndex].reached = reached;
		problems.problemList[problemIndex].victory = !tooLate;
	}
}

void MainWidget::nextProblemInit() {
	problemIndex++;
	int pbTotal=problems.problemList.size();
	
	if(problemIndex == pbTotal - 1) {
		pbplay->setText(tr("Score"));
	}
	
	if(problemIndex>=pbTotal) {
		setTileNumber(selection::nb_tiles);	//init back the interface
		problemStart = true;
		pbplay->setText(tr("Play"));
		
		//display the score result
		textEditC->setPlainText(problems.scoreResult());
		return;
	}
	
	//access to the problem parameters
	Param p = problems.problemList[problemIndex];
	
	//set tiles number
	int n = (p.tileList).size();
	if(n>=3 && n<=6) {
		selection::nb_tiles=n;
	} else {
		selection::nb_tiles=6;
	}
	
	//check the correct action in the Tile menu
	//this is needed to keep the application in a valid state even if we quit the training mode
	QList<QAction *> actionList = tileGroup->actions();
	int i;
	for(i=0; i<actionList.size(); i++) {
		//check the action of index selection::nb_tiles-3, and uncheck all the others
		actionList[i]->setChecked(i==selection::nb_tiles-3);
	}
	
	//set thinking time
	ClockWidget::nb_sec = p.roundTime;
	
	//set target
	target = p.target;
	board->initTarget(p.target);
	
	//init interface, timer and variables
	board->updateTileNumber();
	beginGame();
	
	//if there was an error in the file, display the debug message
	if(!p.debugMessage.isEmpty()){
		textEditC->insertPlainText(tr("Debug : "));
		textEditC->insertPlainText(p.debugMessage);
		textEditC->insertPlainText("\n\n");
	}
	
	//display problemIndex
	textEditC->insertPlainText(tr("Problem %1/%2").arg(problemIndex+1).arg(pbTotal));
	textEditC->insertPlainText("\n\n");
	
	//display comment before the problem
	textEditC->insertPlainText(p.commentBefore);
	
	//set tiles and tirage
	t.tab.clear();
	for(i=0; i<n; i++) {
		t.tab[((p.tileList)[i])]++;
	}
	board->initSelection(t);
	
	//update display
	board->updateTileDisplay();
	board->updateTargetDisplay();
	
	//start the game by launching the computation of the solutions
	launchComputation();
}

void MainWidget::endGame() {
	emit lockModule( true );
	
	//unlock menus
	modeMenu->setEnabled(true);
	pbMode->setEnabled(true);
	if(global::currentMode!=global::ModeTraining){
		fileMenu->setEnabled(true);
		tileMenu->setEnabled(true);
	}
	
	if(global::currentMode == global::ModeTV) {
		//we can set tiles only in Tv mode 
		emit variableTile( true );
		pbRes->setEnabled(true);
	} else {
		//solve button is not available in challenge mode until game starts		
		pbRes->setEnabled(false);
	}
	
	reinit_SelectionTiles(false);
	pbplay->setEnabled(true);
	clock->stopClock();
	gameRunning=false;
	displaySolutions();
}

//solve the problem if not already done and display solutions
void MainWidget::solve() {
	if(gameRunning){
		endGame();
		defeat();
	} else {
		//this case is possible only in TV mode
		w_solution->init(); //clean the solution module
		launchComputation();
	}
}

//new random problem
//TODO : locks
void MainWidget::modeRandom_game(){
	//init interface, timer and variables
	beginGame();
	
	//buttons are locked during computation
	pbRes->setEnabled( false );
	pbplay->setEnabled( false );
	setCursor( QCursor(Qt::WaitCursor));		//mouse is a waiting cursor during computation
	
	board->randomSelection();
	t = board->t;
	
	if(selection::possible_target){	//the player doesn't like impossible problems
		targetCompute = true;
		J->init(t, 1, 999);
	} else {
		//random target and impossible problems are authorized
		target=selection::minimalTarget+rand()%(selection::maximalTarget-selection::minimalTarget+1);
		board->assignTarget(target);
		J->init(t, target, target);
	}
	
	J->start();	//launch analysis as a separate thread
}

//initialize back the application state
void MainWidget::initialize(){
	//application state
	gameRunning=false;
	tooLate=false;
	
	//texts and images
	reinit_status();
	textEditC->setPlainText("");
	w_solution->init();
	reinit_SelectionTiles(false);
	pbplay->setText(tr("Play"));

	//progress bar of elapsed time
	clock->stopClock();
	clock->initClock();
	
	//target : initialized to value (targetType minimal+targetType maximal+1)/2
	//(+1 because we adjust to the above value)
	int tirage_par_defaut=(selection::minimalTarget+selection::maximalTarget+1)/2;
	target = tirage_par_defaut;	
	board->initTarget(tirage_par_defaut);
	
	//selection
	//t.tab.clear();
	t.initWithPossibleTiles();
	board->initSelection(t);
	
	//display
	board->updateTileDisplay();
	board->updateTargetDisplay();
	
	//tiles are adjustable
	if(global::currentMode == global::ModeTV) {
		//we can set tiles only in Tv mode 
		emit variableTile( true );
	}
	emit lockModule( true );
	pbplay->setEnabled( true );
	
	fileMenu->setEnabled(true);
	tileMenu->setEnabled(true);
	modeMenu->setEnabled(true);
	pbMode->setEnabled(true);
}

/*-------------------------------------------------------------------------------------------------
// launch and stop computation
//-----------------------------------------------------------------------------------------------*/

//launch thread to compute solutions
void MainWidget::launchComputation(){
	//TODO: lock click on selection tiles
	
	//buttons are locked during computation
	pbRes->setEnabled( false );
	pbplay->setEnabled( false );
	
	//display
	setCursor( QCursor(Qt::WaitCursor));		//mouse is a waiting cursor during computation
	
	//computation
	J->init(t, target, target);
	J->start();	//launch launchAnalysis as a separate thread
}

//called when computation stops (connected to J.finished)
void MainWidget::computationFinished() {
	if(targetCompute) {
		targetCompute = false; //initialize back targetCompute
		treatRandomTarget();
	}
	
	//computation has finished : set back mouse cursor
	setCursor( QCursor(Qt::ArrowCursor));
	
	if(gameRunning){
		//update display
		board->updateTargetDisplay();
		board->updateTileDisplay();
		
		//indicate the best possible solutions to the solving widget
		int dist=J->distanceToSolution();
		w_solution->targetRange.first=J->lim_inf-dist;	//J->lim_inf is the studied target
		w_solution->targetRange.second=J->lim_inf+dist;
		
		//unlock solving module
		emit lockModule(false);
		reinit_SelectionTiles(true);
		
		clock->startClock();		//launch countdown
	} else {
		displaySolutions();
		foxAfterComputation();
		//unlock buttons
		pbplay->setEnabled(true);
	}
	
	//unlock buttons
	pbRes->setEnabled(true);
}

void MainWidget::treatRandomTarget() {
	int nb_good_targets=0;
	vector<bool> good_targets=J->reachedTarget();
	int i,j;
	//check if this selection provide a good target
	for(i=selection::minimalTarget;i<=selection::maximalTarget;i++){
		if(good_targets[i]){
			nb_good_targets++;
		}
	}
	if(nb_good_targets>0){	//yes : choose randomly a good target
		int index=1+rand()%nb_good_targets;
		i=selection::minimalTarget-1; j=0;
		while(j<index){
			i++;
			if(good_targets[i]){
				j++;
			}
		}
		target=i;
	} else{
		textEditC->setPlainText(QString(tr("There is no exact solution. Try to set a wider range for minimal and maximal targets.\n")));
		//we choose a random target in the authorized range
		target=selection::minimalTarget+rand()%(selection::maximalTarget-selection::minimalTarget+1);
		//we need to initialize the lim
		J->lim_inf_inf=0;
		J->lim_inf_sup=100000;
		J->lim_sup_inf=0;
		J->lim_sup_sup=100000;
	}
	
	board->assignTarget(target);
	
	//restrain the solutions to the target
	J->lim_inf = target;
	J->lim_sup = target;
	J->deleteOutOfRange();
}


/*-------------------------------------------------------------------------------------------------
// Update display
//-----------------------------------------------------------------------------------------------*/

//paint all tiles with white
void MainWidget::reinit_SelectionTiles(bool clickPossible){
	int value=1;
	if(clickPossible) value=0;
	for(int i=0;i<6;i++){
		emit changeStatus(i,value);
	}
}

//update status, fox and text
void MainWidget::reinit_status() {
	pixmapLabelFox->setPixmap(*normalFox);
	foxText->setText( foxTextDefault );
}

/*-------------------------------------------------------------------------------------------------
// Display results
//-----------------------------------------------------------------------------------------------*/

//player has reached the target or best value in the solving module
void MainWidget::victory(){
	endGame();
	reached = true;
	
	if(!tooLate){
		//the player gets one point only in random mode
		if(global::currentMode==global::ModeRandom){
			w_player->addWin();	
		}
		foxAfterGame(true);
	}
}

//time has elapsed before the player found a solution
void MainWidget::defeat(){
	//the player loses a point only the first time we enter this function and only in random mode
	if(!tooLate && global::currentMode==global::ModeRandom){
		w_player->addLoss();
	}
	
	tooLate=true;
	foxAfterGame(false);
}

//display solutions (on the right)
void MainWidget::displaySolutions() {
	//selection and target
	textEditC->setPlainText("");
	textEditC->insertPlainText(J->displayProblem());
	
	//comment (used only in training mode)
	if(global::currentMode==global::ModeTraining) {
		QString msg = problems.problemList[problemIndex].commentAfter;  //TODO : line of code difficult to understand
		if(!msg.isEmpty()) {
			textEditC->insertPlainText(msg);
			textEditC->insertPlainText(QString("\n\n"));
		}
	}
	
	//solutions
	textEditC->insertPlainText(J->displayBase());
}

//update fox image after a computation
void MainWidget::foxAfterComputation(){
	if(J->distanceToSolution()==0){
		foxText->setText( tr("Solvable !") );
		pixmapLabelFox->setPixmap(*happyFox);
	}
	else{
		foxText->setText( tr("Unsolvable.") );
		pixmapLabelFox->setPixmap(*sadFox);
	}
}

//update fox image after game is finished
void MainWidget::foxAfterGame(bool wonGame){
	if(wonGame){
		foxText->setText( tr("You win !") );
		pixmapLabelFox->setPixmap(*happyFox);
	}
	else{
		foxText->setText( tr("You lose.") );
		pixmapLabelFox->setPixmap(*sadFox);
	}
}

/*-------------------------------------------------------------------------------------------------
// Functions called to adjust the tiles
//-----------------------------------------------------------------------------------------------*/

//called when the target has changed (after a tile was clicked)
void MainWidget::targetChanged(int newTarget) {
	target = newTarget;
	reinit_SelectionTiles( false );
	reinit_status();
}

//called everytime a selection tile changes
void MainWidget::selectionChanged(selection newSelection) {
	t = newSelection;
	reinit_SelectionTiles( false );
	reinit_status();
}

//Open file Dialog Box
void MainWidget::problemFileOpen() {
	//reinit the problem list
	problems.problemList.clear();
	
	QString filename = QFileDialog::getOpenFileName(this, tr("Problem files"), "", "*.kts");
	if ( !filename.isEmpty() ) {
		setCursor( QCursor(Qt::WaitCursor) );
		problems.readXmlFile(filename);
		unsetCursor();
	}
}

void MainWidget::changeMode(int newMode) {
	//if this is the same Mode than before, nothing happens
	if(global::currentMode == newMode) return;
	
	//confirmation if during training
	if(global::currentMode==global::ModeTraining && !problemStart) {
		int ret = QMessageBox::warning (this, tr("Quit current game"), tr("Do you really want to stop the current training game ? "), QMessageBox::Ok|QMessageBox::Cancel);
		if(ret!=QMessageBox::Ok) {
			trainingAction->setChecked(true); //check back the training action
			return;
		} else {
			//init back the interface and the parameters
			setTileNumber(selection::nb_tiles);
			problemStart = true;
			pbplay->setText(tr("Play"));
		}
	}
	
	global::currentMode = newMode;
	if(global::currentMode==global::ModeTV) {
		emit variableTile( true );
		pbRes->setEnabled(true);
		w_player->setEnabled(false);
		pbMode->setIcon(*television);
		tvAction->setChecked(true);
		textEditC->setVisible(true);
		textEdit2->setVisible(false);
		textEditC->setPlainText(tr("TV Mode\n\nChange the tiles yourself."));
	}
	
	if(global::currentMode==global::ModeRandom) {
		emit variableTile( false );
		pbRes->setEnabled(false);
		w_player->setEnabled(true);
		pbMode->setIcon(*dice);
		challengeAction->setChecked(true);
		textEditC->setVisible(true);
		textEdit2->setVisible(false);
		textEditC->setPlainText(tr("Challenge Mode"));
	}
	
	if(global::currentMode==global::ModeTraining) {
		emit variableTile( false );
		pbRes->setEnabled(false);
		w_player->setEnabled(false);
		pbMode->setIcon(*tick);
		trainingAction->setChecked(true);
		fileMenu->setEnabled(false);
		tileMenu->setEnabled(false);
		textEdit2->setVisible(true);
		textEditC->setVisible(false);
		textEdit2->setHtml(tr("<p>Training mode.</p><p>Play with problems saved in files.</p><p>Problems can be found on <a href=\"http://kitsune.tuxfamily.org/wiki/doku.php?id=problem_files\">kitsune web site</a>.</p>"));
	} else {
		fileMenu->setEnabled(true);
		tileMenu->setEnabled(true);
	}
}

void MainWidget::rotateMode() {
	if(global::currentMode==global::ModeTV) {
		changeMode(global::ModeTraining);
	} else
	if(global::currentMode==global::ModeTraining) {
		changeMode(global::ModeRandom);
	} else
		changeMode(global::ModeTV);
}

void MainWidget::setTileNumber(int tileNumber) {
	if(tileNumber >= OptionsWidget::min_nb_tiles && tileNumber <= OptionsWidget::max_nb_tiles) {
		selection::nb_tiles = tileNumber;
		
		if(OptionsWidget::saveOptions) {
			QString optionsFilePath=QDir::homePath()+ QString("/.kitsune.options.") + global::kitsuneVersion;
			QSettings settings(optionsFilePath, QSettings::IniFormat);
			settings.setValue("tileNumber", selection::nb_tiles);
		}
		
		OptionsWidget::setGameRule();
		board->updateTileNumber();
		initialize();
	}
}

void MainWidget::selectTiles() {
	SelectionWidget selectionWidget(this);
	selectionWidget.exec();
	
	//if the current selection is not included in the new authorized tiles
	//then we need to reinit the displayed tiles
	selection authorizedSelection;
	authorizedSelection.tab = selection::ref;
	if(!t.isIncludedIn(authorizedSelection)) {
		t.initWithPossibleTiles();
		board->initSelection(t);
		board->updateTileDisplay();
	}
}
