/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MAINWIDGET_H
#define MAINWIDGET_H

#include "selection.h"
#include "game.h"
#include "popupwidget.h"
#include "tilewidget.h"
#include "boardwidget.h"
#include "solutionwidget.h"
#include "optionswidget.h"
#include "global.h"
#include "clockwidget.h"
#include "problem.h"
#include "playerwidget.h"

#include <QMainWindow>
#include <QWidget>
#include <QString>

class QSignalMapper;
class QTextBrowser;
class QTextEdit;
class QLabel;
class QPixmap;
class QMenu;
class QActionGroup;

class MainWidget : public QMainWindow, KitsuneFonts {
	Q_OBJECT
	
  public:
	MainWidget( QWidget *parent=0 );
	
	QString kitsuneVersionText, foxTextDefault;
	
	//class game must not be destroyed after solve (active thread)
	game *J;
	
	//parameters describing a given problem
	selection t;
	targetType target;
	
	//parameters describing application state
	//true if user is trying to solve a problem in limited time
	bool gameRunning;
	//true if player didn't find a solution in the given time
	bool tooLate;
	
	bool targetCompute; //indicate we are computing a reachable target
	
	QActionGroup *tileGroup;
	QAction *tvAction, *challengeAction, *trainingAction;
	QMenu *fileMenu, *tileMenu, *modeMenu, *infoMenu;
	
	//parent widgets
	QWidget *hbox_res;
	QTextEdit *textEditC;
	QTextBrowser *textEdit2;
	
	BoardWidget *board;
	SolutionWidget *w_solution;
	
	//widgets
	QLabel *foxText;
	ClockWidget *clock;
	
	QPushButton *pbplay, *pbRes;
	
	//image du renard
	QLabel *pixmapLabelFox;
	QPixmap *normalFox, *happyFox, *sadFox;
	
	//pictures for game modes
	QPixmap *television, *dice, *tick;
	QPushButton *pbMode;
	
	//mapper for the "read a text file" signal
	QSignalMapper *readText_Mapper, *tileNumber_Mapper, *mode_Mapper;
	
	PlayerWidget *w_player;
	Problem problems;
	int problemIndex;
	bool problemStart;
	bool reached;

  public slots:
	//open menus
	void options();
	void selectTiles();
	void readText(int num);
	
	//functions called when a player push a button
	void changeMode(int);
	void rotateMode();
	void play();
	void solve();
	void beginGame();
	void endGame();
	void modeTV_game();
	void modeRandom_game();
	void modeTraining_game();
	void initialize();
	void saveResult();
	void nextProblemInit();
	void treatRandomTarget();
	
	//start/end of a computation
	void launchComputation();
	void computationFinished();
	
	//update display
	void reinit_status();
	void reinit_SelectionTiles(bool);
	
	//display results
	void victory();
	void defeat();
	void displaySolutions();
	void foxAfterComputation();
	void foxAfterGame(bool);
	
	//called when target or selection changed in the widgetboard
	void targetChanged(int);
	void selectionChanged(selection);
	
	//read a problem file
	void problemFileOpen();
	
	//set tile number
	void setTileNumber(int);
	
  signals :
	void variableTile(bool);
	void changeStatus(int, int);
	void lockModule(bool);	//lock/unlock the solution widget
};

#endif
