/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "operationwidget.h"
#include "selection.h"

#include <QtGui>

//conventional values to identify the 4 elementary operations
//values must be consecutives (to write loops)
const int OperationWidget::Plus=1;
const int OperationWidget::Minus=2;
const int OperationWidget::Multiply=3;
const int OperationWidget::Divide=4;

//initial value of static pointers
QPixmap * OperationWidget::pixPlus=0;
QPixmap * OperationWidget::pixMinus=0;
QPixmap * OperationWidget::pixMultiply=0;
QPixmap * OperationWidget::pixDivide=0;
QPixmap * OperationWidget::pixEmpty=0;

//Create a widget with an operation button
//numW : id which must be unique for all value tiles/operation tiles
//opW : operation type (Plus, Minus, Multiply, Divide)
//clickPossibleW : clickable or not
OperationWidget::OperationWidget( QWidget *parent, int numW, int opW, bool clickPossibleW)
        : QWidget( parent) {
	
	//init pixmap if necessary (only once)
	if(!pixPlus) {
		pixPlus   = new QPixmap( QString(":/images/big_plus.png") );
	}
	if(!pixMinus) {
		pixMinus  = new QPixmap( QString(":/images/big_minus.png") );
	}
	if(!pixMultiply) {
		pixMultiply   = new QPixmap( QString(":/images/big_multiply.png") );
	}
	if(!pixDivide) {
		pixDivide = new QPixmap( QString(":/images/big_divide.png") );
	}
	if(!pixEmpty) {
		pixEmpty   = new QPixmap( QString("") );
	}
	
	//init widget index
	//num must be different for each tiles
	num=numW;
	clickPossible=clickPossibleW;
	
	QVBoxLayout *vboxLayout = new QVBoxLayout( this );
	vboxLayout->setSpacing(0);
	vboxLayout->setMargin(0);
	
	//create a pushbutton if widget is clickable, or a label otherwise
	if(clickPossible){
		pbOperation = new QPushButton( "", this);
		pbOperation->setFlat( true );
		pbOperation->setFixedSize( 32, 32 );
		pbOperation->setFocusPolicy(Qt::NoFocus);
		connect( pbOperation, SIGNAL(clicked()), this, SLOT(mousePressEvent()) );
		vboxLayout->addWidget( pbOperation );
	} else{
		lbOperation = new QLabel(this);
		lbOperation->setFixedSize( 32, 32 );
		lbOperation->setAlignment( Qt::AlignHCenter );
		vboxLayout->addWidget( lbOperation );
	}
	
	//display image
	changeOperation(numW, opW);
}

//called when the operation tile is clicked
void OperationWidget::mousePressEvent() {
	emit value(num, op);
}

//update diplayed image
void OperationWidget::changeOperation(int numW, int opW) {
	if(num!=numW) return;
	op=opW;
	QPixmap* pix=pixEmpty;
	
	if(op==Plus) {
		pix=pixPlus;
	} else
	if(op==Minus) {
		pix=pixMinus;
	} else
	if(op==Multiply) {
		pix=pixMultiply;
	} else
	if(op==Divide) {
		pix=pixDivide;
	}
	
	if(clickPossible){
		pbOperation->setIcon( *pix );
		pbOperation->setIconSize( QSize(28,28) );
	} else{
		lbOperation->setPixmap( *pix );
	}
}

//clean pixmap pointers
//WARNING  : this function must be called when Qt application is restarted (after a language change for example)
void OperationWidget::clearPixmapPointers() {
	pixPlus=0;
	pixMinus=0;
	pixMultiply=0;
	pixDivide=0;
	pixEmpty=0;
}
