/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "optionswidget.h"

//static variables in calcul and ClockWidget class
#include "calculation.h"
#include "clockwidget.h"
#include "selection.h"
#include "operationwidget.h"
#include "selectionwidget.h"

#include <QtGui>
#include <fstream>

//static variables initialization
int OptionsWidget::min_roundTime=10;
int OptionsWidget::max_roundTime=300;
int OptionsWidget::min_nb_tiles=3;
int OptionsWidget::max_nb_tiles=6;
int OptionsWidget::min_min_targetType=1;
int OptionsWidget::max_min_targetType=100;
int OptionsWidget::min_max_targetType=1;
int OptionsWidget::max_max_targetType=999;

int OptionsWidget::NewLanguage=5;  //value to restart the program (any value but 0 is ok)

//get system local language at program launch
//return something like fr_FR (2 characters for language -ISO 639- and 2 for country -ISO 3166-) and keep only fr
QString OptionsWidget::language=QLocale::system().name().left(2);

bool OptionsWidget::saveOptions=false;

//Options widget
OptionsWidget::OptionsWidget(QWidget* parent) : QDialog( parent) {
	setWindowTitle(QString(tr("Options")));
	
	QGroupBox *simplif = new QGroupBox(tr("Simplifications"), this);
	//check buttons for canonizations
	QLabel *canoTitle1 = new QLabel(simplif);
	canoTitle1->setText(tr("Use \"2+2=22\""));
	canoTitle1->setToolTip(tr("Use \"2+2=22\" property to reduce the number of solutions"));
	cano1 = new QCheckBox(simplif);
	
	QLabel *canoTitle2 = new QLabel(simplif);
	canoTitle2->setText(tr("Use \"optimal intermediate results\""));
	canoTitle2->setToolTip(tr("Don't display solutions whose part is a solution itself"));
	cano2 = new QCheckBox(simplif);
	
	QLabel *canoTitle3 = new QLabel(simplif);
	canoTitle3->setText(tr("Use \"factorization\""));
	canoTitle3->setToolTip(tr("Consider factorized and developed form of a calculus as equivalent"));
	cano3 = new QCheckBox(simplif);
	
	QGridLayout *simplifLayout= new QGridLayout( simplif );
	simplifLayout->addWidget(canoTitle1, 0, 0);
	simplifLayout->addWidget(cano1, 0, 1, 1, 1, Qt::AlignRight);
	simplifLayout->addWidget(canoTitle2, 1, 0);
	simplifLayout->addWidget(cano2, 1, 1, 1, 1, Qt::AlignRight);
	simplifLayout->addWidget(canoTitle3, 2, 0);
	simplifLayout->addWidget(cano3, 2, 1, 1, 1, Qt::AlignRight);

	QGroupBox *rulesBox = new QGroupBox(tr("Game rules for %1 tiles").arg(selection::nb_tiles), this);
	
	//round length
	QLabel *titreTemps = new QLabel(rulesBox);
	titreTemps->setText(tr("Round length"));
	boxTime = new QSpinBox(rulesBox);
	boxTime->setRange(min_roundTime,max_roundTime);
	boxTime->setFixedSize(boxTime->sizeHint());	
	
	//minimal target
	QLabel *titretargetType = new QLabel(rulesBox);
	titretargetType->setText(tr("Minimal target"));
	boxtargetType = new QSpinBox(rulesBox);
	boxtargetType->setRange(min_min_targetType,max_min_targetType);
	boxtargetType->setFixedSize(boxtargetType->sizeHint());
	
	//Maximal target
	QLabel *titretargetType2 = new QLabel(rulesBox);
	titretargetType2->setText(tr("Maximal target"));
	boxtargetType2 = new QSpinBox(rulesBox);
	boxtargetType2->setRange(min_max_targetType,max_max_targetType);
	boxtargetType2->setFixedSize(boxtargetType2->sizeHint());
	
	//Decide if target is necessary reachable
	QLabel *title_pt = new QLabel(rulesBox);
	title_pt->setText(tr("Reachable target only"));
	title_pt->setToolTip(tr("Generates problems with reachable target only"));
	possible_target = new QCheckBox(rulesBox);
	
	QGridLayout *rulesBoxLayout= new QGridLayout( rulesBox );
	rulesBoxLayout->addWidget(titreTemps, 0, 0);
	rulesBoxLayout->addWidget(boxTime, 0, 1);
	rulesBoxLayout->addWidget(titretargetType, 1, 0);
	rulesBoxLayout->addWidget(boxtargetType, 1, 1);
	rulesBoxLayout->addWidget(titretargetType2, 2, 0);
	rulesBoxLayout->addWidget(boxtargetType2, 2, 1);
	rulesBoxLayout->addWidget(title_pt, 3, 0);
	rulesBoxLayout->addWidget(possible_target, 3, 1);
	
	QGroupBox *grouplanguage = new QGroupBox(tr("Language"), this);
	//radio buttons to choose language
	QLabel *languageTitle1 = new QLabel(grouplanguage);
	languageTitle1->setText("Franais");
	language1 = new QRadioButton(grouplanguage);
	
	QLabel *languageTitle2 = new QLabel(grouplanguage);
	languageTitle2->setText("English");
	language2 = new QRadioButton(grouplanguage);
	
	QLabel *languageTitle4 = new QLabel(grouplanguage);
	languageTitle4->setText("Deutsch");
	language4 = new QRadioButton(grouplanguage);
	
	QLabel *languageTitle3 = new QLabel(grouplanguage);
	//unicode string of "japanese" in japanese
	ushort carac1=26085;
	ushort carac2=26412;
	ushort carac3=35486;
	QString japanese;
	japanese.append(carac1);
	japanese.append(carac2);
	japanese.append(carac3);
	languageTitle3->setText(japanese);
	language3 = new QRadioButton(grouplanguage);
	
	QGridLayout *languageLayout= new QGridLayout;
	languageLayout->addWidget(languageTitle1, 0, 0);
	languageLayout->addWidget(language1, 0, 2, 1, 1, Qt::AlignRight);
	languageLayout->addWidget(languageTitle2, 1, 0);
	languageLayout->addWidget(language2, 1, 2, 1, 1, Qt::AlignRight);
	languageLayout->addWidget(languageTitle4, 2, 0);
	languageLayout->addWidget(language4, 2, 2, 1, 1, Qt::AlignRight);
	languageLayout->addWidget(languageTitle3, 3, 0);
	languageLayout->addWidget(language3, 3, 2, 1, 1, Qt::AlignRight);
	grouplanguage->setLayout(languageLayout);
	
	//checkbox to save options on the disk
	QWidget *saveHbox = new QWidget( this );
	QHBoxLayout *hlayout1 = new QHBoxLayout( saveHbox );
	saveOptionsBox = new QCheckBox( saveHbox );
	QLabel *saveOptionsTitle = new QLabel( saveHbox );
	saveOptionsTitle->setText( tr("Save options and score") );
	hlayout1->addWidget( saveOptionsTitle );
	hlayout1->addStretch(1);
	hlayout1->addWidget( saveOptionsBox );
	hlayout1->setSpacing(1);
	hlayout1->setMargin(1);
	
	QWidget *buttonsHbox = new QWidget( this );
	QHBoxLayout *hlayout2 = new QHBoxLayout( buttonsHbox );
	
	//default options
	QPushButton *pbDefault = new QPushButton( tr("Default options"), buttonsHbox );
	pbDefault->setFont( font_normal );
	connect( pbDefault, SIGNAL( clicked() ), this, SLOT( setDefault() ) );
	
	//"OK" button to close window
	QPushButton *pbOk = new QPushButton( tr("OK"), buttonsHbox );
	pbOk->setFont( font_normal );
	pbOk->setFocus();
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( modifOk() ) );
	
	hlayout2->addWidget( pbDefault );
	hlayout2->addStretch(1);
	hlayout2->addWidget( pbOk );
	hlayout2->setSpacing(1);
	hlayout2->setMargin(1);
	
	//widget positions
	QVBoxLayout *vlayout = new QVBoxLayout( this );
	
	vlayout->addWidget(simplif);
	vlayout->addWidget(rulesBox);
	vlayout->addWidget(grouplanguage);
	vlayout->addWidget(saveHbox);
	vlayout->addWidget(buttonsHbox);
	
	//state initialization
	if (calcul::cano1_active) {
		cano1->setCheckState(Qt::Checked);
	} else {
		cano1->setCheckState(Qt::Unchecked);
	}
	
	if (calcul::cano2_active) {
		cano2->setCheckState(Qt::Checked);
	} else {
		cano2->setCheckState(Qt::Unchecked);
	}
	
	if (calcul::cano3_active) {
		cano3->setCheckState(Qt::Checked);
	} else {
		cano3->setCheckState(Qt::Unchecked);
	}
	
	if (selection::possible_target) {
		possible_target->setCheckState(Qt::Checked);
	} else {
		possible_target->setCheckState(Qt::Unchecked);
	}
	
	if (language=="fr") {
		language1->setDown(true);
	}
	if (language=="en") {
		language2->setDown(true);
	}
	if (language=="de") {
		language4->setDown(true);
	}
	if (language=="ja") {
		language3->setDown(true);
	}
	
	boxTime->setValue(ClockWidget::nb_sec);
	boxtargetType->setValue(selection::minimalTarget);
	boxtargetType2->setValue(selection::maximalTarget);
	
	if(saveOptions) {
		saveOptionsBox->setCheckState(Qt::Checked);
	} else {
		saveOptionsBox->setCheckState(Qt::Unchecked);
	}
}

//modifications are validated
void OptionsWidget::modifOk() {
	//parameters update
	calcul::cano1_active=cano1->isChecked();
	calcul::cano2_active=cano2->isChecked();
	calcul::cano3_active=cano3->isChecked();
	ClockWidget::nb_sec=boxTime->value();
	
	if(boxtargetType->value()<boxtargetType2->value()){
		//normal case : minimal value is smaller than maximal value
		selection::minimalTarget=boxtargetType->value();
		selection::maximalTarget=boxtargetType2->value();
	} else {
		//reverse the minimal and maximal value
		selection::minimalTarget=boxtargetType2->value();
		selection::maximalTarget=boxtargetType->value();
	}
	
	selection::possible_target=possible_target->isChecked();
	
	//language parameter
	if(language1->isChecked() || language2->isChecked() || language3->isChecked() || language4->isChecked() || language5->isChecked()) {
		if(language1->isChecked()) {
			language="fr";
		} else if (language2->isChecked()) {
			language="en";
		} else if (language3->isChecked()) {
			language="ja";
		} else if (language4->isChecked()) {
			language="de";
		}
	}
	
	//check whether to save or not options file
	saveOptions=saveOptionsBox->isChecked();
		
	if(saveOptions) {
		QString optionsFilePath=QDir::homePath()+ QString("/.kitsune.options.") + global::kitsuneVersion;
		QSettings settings(optionsFilePath, QSettings::IniFormat);
		
		settings.setValue("version", global::kitsuneVersion);
		settings.setValue("cano/twoBytwo", calcul::cano1_active);
		settings.setValue("cano/intermediate", calcul::cano2_active);
		settings.setValue("cano/factorization", calcul::cano3_active);
		settings.setValue("tileNumber", selection::nb_tiles);
		
		settings.beginGroup(QString("rules%1").arg(selection::nb_tiles));
		settings.setValue("roundLength", ClockWidget::nb_sec);
		settings.setValue("minTarget", selection::minimalTarget);
		settings.setValue("maxTarget", selection::maximalTarget);
		settings.setValue("possibleTarget", selection::possible_target);
		settings.endGroup();
		
		settings.setValue("language", language);
	} else {
		//delete options file
		deleteOptionsFile();
	}
	
	//restart kitsune if language changed
	//note : qApp is a Qt symbol referencing the entire application
	if(language1->isChecked() || language2->isChecked() || language3->isChecked() || language4->isChecked() || language5->isChecked()) {
		qApp->exit(NewLanguage);
	}
	
	//close options dialog
	close();
}

//set default options
void OptionsWidget::setDefault() {
	cano1->setCheckState(Qt::Checked);
	cano2->setCheckState(Qt::Checked);
	cano3->setCheckState(Qt::Checked);
	
	setDefaultGameRule();
	boxTime->setValue(ClockWidget::nb_sec);
	boxtargetType->setValue(selection::minimalTarget);
	boxtargetType2->setValue(selection::maximalTarget);
	if (selection::possible_target) {
		possible_target->setCheckState(Qt::Checked);
	} else {
		possible_target->setCheckState(Qt::Unchecked);
	}
}

//this function is called in kitsune.cpp before creating the main widget
void OptionsWidget::readFile() {
	//access to the settings stored in an ini file
	QString optionsFilePath=QDir::homePath()+ QString("/.kitsune.options.") + global::kitsuneVersion;
	QSettings settings(optionsFilePath, QSettings::IniFormat);
	
	QString version = settings.value("version").toString();
	if(version != global::kitsuneVersion) {
		//settings don't exist
		saveOptions = false;
		selection::nb_tiles = 6;
		setDefaultGameRule();
		//default language : en
		if(language!="en" && language!="fr" && language!="ja" && language!="de"){	//it happens if Locale isn't one of these 3
			language="en";
		}
		return;
	}
	
	saveOptions = true;
	
	calcul::cano1_active = settings.value("cano/twoBytwo").toBool();
	calcul::cano2_active = settings.value("cano/intermediate").toBool();
	calcul::cano3_active = settings.value("cano/factorization").toBool();
	
	//tile number
	int tileNumber = settings.value("tileNumber").toInt();
	if( tileNumber>=min_nb_tiles && tileNumber<=max_nb_tiles) {
		selection::nb_tiles=tileNumber;
	}
	else {
		//QMessageBox::warning (0, tr("Options"), tr("Incorrect options file."), QMessageBox::Ok);
	}
	
	//read language
	language = settings.value("language").toString();
	
	setGameRule();
}

//delete options file
void OptionsWidget::deleteOptionsFile() {
	QString optionsFilePath=QString("/.kitsune.options.").append(global::kitsuneVersion);
	QFile f_options(QDir::homePath()+optionsFilePath);
	
	//nothing to do if file doesn't exist
	if (!f_options.exists()) {
		return;
	}
	
	if(f_options.remove()) {
		QMessageBox::information (this, tr("Deletion"), tr("The options file has been deleted."), QMessageBox::Ok);
	
	} else {
		//TODO : indicate the file path in the error message
		QMessageBox::warning (this, tr("Deletion"), tr("Deletion of the options file has failed."), QMessageBox::Ok);
	}
}

//set the game rules
void OptionsWidget::setGameRule() {
	//use the default rule is we don't use  an options file
	if(!saveOptions) {
		setDefaultGameRule();
		return;
	}
	
	//access to the settings stored in an ini file	
	QString optionsFilePath=QDir::homePath()+ QString("/.kitsune.options.") + global::kitsuneVersion;
	QSettings settings(optionsFilePath, QSettings::IniFormat);

	QString version = settings.value("version").toString();
	if(version != global::kitsuneVersion) {
		//settings don't exist : use the default game rules
		setDefaultGameRule();
		return;
	}

	settings.beginGroup(QString("rules%1").arg(selection::nb_tiles));
	int value_4 = settings.value("roundLength").toInt();
	int value_6 = settings.value("minTarget").toInt();
	int value_7 = settings.value("maxTarget").toInt();
	bool value_8 = settings.value("possibleTarget").toBool();
	settings.endGroup();

	//check values
	if( value_4>=min_roundTime && value_4<=max_roundTime 
		&& value_6>=min_min_targetType && value_6<=max_min_targetType 
		&& value_7>=min_max_targetType && value_7<=max_max_targetType
		&& value_6<=value_7) {
		ClockWidget::nb_sec=value_4;
		selection::minimalTarget=value_6;
		selection::maximalTarget=value_7;
		selection::possible_target=value_8;
	}
	else {
		//QMessageBox::warning (0, tr("Options"), tr("Incorrect options file."), QMessageBox::Ok);
		setDefaultGameRule();
	}
	
	//authorized Tiles
	settings.beginGroup(QString("authorizedTiles"));
	if(settings.contains("tile0")) {
		//authorized tiles have been defined, initialize selection::ref with them
		selection::ref.clear();
		int i;
		for(i=0; i<24; i++) {
			int a = settings.value(QString("tile%1").arg(i)).toInt();
			tile oneTile = (tile) a;
			
			map<tile, int>::iterator it = selection::ref.find(oneTile);
			if( it != selection::ref.end()) {
				it->second = it->second + 1;
			} else {
				selection::ref[oneTile] = 1;
			}
		}
	}
	settings.endGroup();
}

void OptionsWidget::setDefaultGameRule() {
	if(selection::nb_tiles==3) {
		ClockWidget::nb_sec=40;
		selection::minimalTarget=10;
		selection::maximalTarget=150;
		selection::possible_target=true;
	} else
	if(selection::nb_tiles==4) {
		ClockWidget::nb_sec=40;
		selection::minimalTarget=50;
		selection::maximalTarget=500;
		selection::possible_target=true;
	} else
	if(selection::nb_tiles==5) {
		ClockWidget::nb_sec=50;
		selection::minimalTarget=100;
		selection::maximalTarget=999;
		selection::possible_target=true;
	} else {
		ClockWidget::nb_sec=50;
		selection::minimalTarget=100;
		selection::maximalTarget=999;
		selection::possible_target=false;
	}
}
