/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "popupwidget.h"
#include "optionswidget.h"

#include <QtGui>

//The "licence" popup is a bit different of other popups, with two more buttons
//isLicenceWindow indicates if the popup is a licence "popup" or not
PopupWidget::PopupWidget( QWidget* parent, bool isLicenceWindow, QString fileName, QString windowTitle)
	 : QDialog( parent) {
	//window title
	if(windowTitle!=QString("")) {
		setWindowTitle(windowTitle);
	}
		
	//vertical layout
	QVBoxLayout *vboxPopup = new QVBoxLayout( this );
	vboxPopup->setSpacing(0);
	vboxPopup->setMargin(0);
	
	//horizontal layout, one for the text, one for the buttons
	QWidget *horizontalLayout1 = new QWidget( this );
	QWidget *horizontalLayout2 = new QWidget( this );
	QHBoxLayout *hboxLayout1 = new QHBoxLayout(horizontalLayout1);
	QHBoxLayout *hboxLayout2 = new QHBoxLayout(horizontalLayout2);
	
	hboxLayout1->setSpacing(0);
	hboxLayout1->setMargin(0);
	hboxLayout2->setSpacing(0);
	hboxLayout2->setMargin(0);
	
	vboxPopup->addWidget( horizontalLayout1 );
	vboxPopup->addWidget( horizontalLayout2 );
	
	//main text widget
	QTextBrowser *textEditPopup = new QTextBrowser( horizontalLayout1 );
	
	//text widget size (imposing window size)
	textEditPopup->setMinimumWidth(600);
	textEditPopup->setMinimumHeight(400);
	
	//text parameter
	textEditPopup->setReadOnly( TRUE );
	textEditPopup->setOpenExternalLinks( TRUE );	//external links are opened in the system default browser
	
	if(fileName!=QString("")) {
		QFile file(fileName);
		file.open( QIODevice::ReadOnly );
		QTextStream textestream( &file );
		textestream.setCodec("UTF-8");					//all files must be encoded in UTF8
		textEditPopup->setHtml(textestream.readAll());	//read file
		file.close();
	}
	
	hboxLayout1->addWidget(textEditPopup);
	
	//display "Licence" button if necessary
	if (isLicenceWindow){
		QPushButton *pbLicence = new QPushButton( tr("License"), horizontalLayout2 );
		pbLicence->setFont( font_normal );
		connect( pbLicence, SIGNAL( clicked() ), this, SLOT( openLicence() ) );
		
		hboxLayout2->addStretch(1);
		hboxLayout2->addWidget(pbLicence);
		hboxLayout2->addStretch(1);
	}
	
	//Ok button
	QPushButton* pbOk = new QPushButton( tr("OK"), horizontalLayout2 );
	pbOk->setFont( font_normal );
	//Ok button has focus by default, so that pressing "enter" closes the popup
	pbOk->setFocus();
	hboxLayout2->addStretch(1);
	hboxLayout2->addWidget(pbOk);
	hboxLayout2->addStretch(1);
	
	//close window when "ok" is clicked
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( close() ) );
}

//display original english GPL
void PopupWidget::openLicence() {
	PopupWidget popup( this, false, QString(":/txt/en/gpl.html"),
					QString(tr("GNU GPL original (english) license")));
	popup.exec();
}
