/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef PROBLEM_H
#define PROBLEM_H

#include "global.h"
#include "calculation.h"

#include <QWidget>

//class to store the parameters of a problem
class Param {
	public:
	//possible tiles
	vector<tile> tileList;
	
	//target
	targetType target;
	
	//round length
	int roundTime;
	
	//comment before and after the problem
	QString commentBefore;
	QString commentAfter;
	
	//string used to display debug information on the screen
	QString debugMessage;
	
	// --- temporary information
	targetType bestAnswer;	//player's best solution
	bool victory;		//target reached or not during the round length
	bool reached;		//target reached or not
};

//class to store a list of problems (used in training mode for example)
class Problem : QObject {
	Q_OBJECT
	
  public:
	Problem() {};
	
	//list of problems (tiles, target, round length)
	vector<Param> problemList;
	
	//comment before and after the complete list of problems
	QString commentBefore;
	QString commentAfter;
	
	//author and date
	QString author;
	QString date;
	
	//point values when the player reaches the target during and out round length
	int winPoint;
	int reachedPoint;
	
	QString scoreResult();
	QString intro();
	
  public slots:
	void writeXmlFile(QString fileName);
	void readXmlFile(QString fileName);
};

#endif
