/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SELECTION_H
#define SELECTION_H

#include "calculation.h"

#include <map>
#include <vector>

class selection {
  public :
	map<tile, int> tab; //tiles table
	
	static int nb_tiles;
	static int minimalTarget;
	static int maximalTarget;
	static bool possible_target;
	static map<tile, int> ref;	//list of authorized tiles
	static map<tile, int> getDefaultRules();
	
	int size() const;
	bool operator!=(const selection&) const;
	bool operator==(const selection&) const;
	bool operator<(const selection&) const;
	bool isIncludedIn(selection) const;
	selection operator+(const selection&) const;
	
	vector<selection> computeSubSelections() const; //compute the sub-selection sorted in a "correct" order
	int find_inside(const vector<selection>&) const; //find the position of a selection in a list of selections
	
	void initWithPossibleTiles(); //init the tab internal object with authorized tiles
	vector<tile> getTileList();
};

//display a selection
ostream& operator<<(ostream& ostr, const selection& t);

//functions used in mainWidget.cpp to manipulate the tiles
tile leastTileSup(selection,tile);
tile leastTileInf(selection,tile);
tile incrementTile(tile);
tile decrementTile(tile);

#endif
