/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "selectionwidget.h"
#include "tilewidget.h"
#include "optionswidget.h"
#include "global.h"

#include <QtGui>

SelectionWidget::SelectionWidget( QWidget *parent ) : QDialog( parent) {
	setWindowTitle(QString(tr("Authorized tiles")));
	
	QVBoxLayout *vlayout = new QVBoxLayout;
	
	//-----list of authorized tiles-----------
	QWidget *tilesGrid = new QWidget( this );
	QGridLayout *tilesLayout = new QGridLayout( tilesGrid );
	
	map<tile, int>::iterator it;
	int tileId=700;
	int x = 0;
	int y = 0;
	int i=0;
	for(it = selection::ref.begin(); it!=selection::ref.end(); it++) {
		for(i=0; i < it->second; i++) {
			TileWidget *oneTile = new TileWidget(this, tileId, 1);
			connect( oneTile, SIGNAL(change(int, bool)), this, SLOT(selectionTileClicked(int, bool)) );
			connect(this, SIGNAL(changeTile(int, int)), oneTile, SLOT(displayRefresh(int, int)) );
			oneTile->displayRefresh(tileId, it->first);
			tileId++;
			tilesLayout->addWidget(oneTile, x, y);
			y++;
			if(y==5) {
				y=0;
				x++;
			}
		}
	}
	
	//-----default and ok buttons-------------
	QWidget *buttonsHbox = new QWidget( this );
	QHBoxLayout *hlayout = new QHBoxLayout( buttonsHbox );
	
	//default options
	QPushButton *pbDefault = new QPushButton( tr("Default values"), buttonsHbox );
	pbDefault->setFont( font_normal );
	connect( pbDefault, SIGNAL( clicked() ), this, SLOT( modifDefault() ) );
	
	//"OK" button to close window
	QPushButton *pbOk = new QPushButton( tr("OK"), buttonsHbox );
	pbOk->setFont( font_normal );
	pbOk->setFocus();
	connect( pbOk, SIGNAL( clicked() ), this, SLOT( modifOk() ) );
	
	hlayout->addStretch(1);
	hlayout->addWidget( pbDefault );
	hlayout->addStretch(1);
	hlayout->addWidget( pbOk );
	hlayout->addStretch(1);
	
	hlayout->setSpacing(1);
	hlayout->setMargin(1);
	
	//-----main layout--------
	vlayout->addWidget(tilesGrid);
	vlayout->addWidget(buttonsHbox);
	
	setLayout(vlayout);
	
	//initialize the currentChoice object with the list of authorized tiles
	//if "Ok" is cliqued, the authorized tiles will take the value of currentChoice
	currentChoice = selection::ref;
}

void SelectionWidget::selectionTileClicked(int tile_id, bool increase) {
	//warning : the pointer returned by sender() is valid only during this function, and violates the signal-slot concept
	TileWidget * senderTile = (TileWidget *) sender();
	
	int value = senderTile->lcd->intValue();
	int newValue;
	if(increase) {
		newValue = value + 1;
	} else {
		newValue = value - 1;
	}
	
	//authorized range is [1-255]
	if(newValue<1 || newValue > 255) {
		return;
	}
	
	map<tile, int>::iterator it;
	
	//the number of tiles with "value" decreases of one tile
	it = currentChoice.find(value);
	if( it != currentChoice.end()) {
		if(it->second > 1) {
			it->second = it->second - 1;
		} else {
			//erase this tile
			currentChoice.erase(it);
		}
	}
	
	//the number of tiles with "newvalue" increases of one tile
	it = currentChoice.find(newValue);
	if( it != currentChoice.end()) {
		it->second = it->second + 1;
	} else {
		currentChoice[newValue]=1;
	}
	
	senderTile->displayRefresh(tile_id, newValue);
}

void SelectionWidget::modifOk() {
	selection::ref = currentChoice;
	
	if(OptionsWidget::saveOptions) {
		//save the selection in the options file
		QString optionsFilePath=QDir::homePath()+ QString("/.kitsune.options.") + global::kitsuneVersion;
		QSettings settings(optionsFilePath, QSettings::IniFormat);
		settings.beginGroup(QString("authorizedTiles"));
		int i=0;
		map<tile, int>::iterator Ti;
		for(Ti=currentChoice.begin(); Ti!=currentChoice.end(); Ti++) {
			int j;
			//Ti->first is a tile that can be used Ti->second times
			for(j=0; j< (Ti->second); j++, i++) {
				settings.setValue(QString("tile%1").arg(i), Ti->first);
			}
		}
		settings.endGroup();
	}
	
	close();
}

void SelectionWidget::modifDefault() {
	//initialize the list of widget with the default values
	//1, 1, 2, 2, ... 9, 9, 10, 10, 25, 50, 75, 100
	vector<int> v(24);
	int i;
	for(i=0; i<20; i++) {
		v[i]=i/2+1;
	}
	v[20]=25;
	v[21]=50;
	v[22]=75;
	v[23]=100;
	
	for(i=0; i<24; i++) {
		emit changeTile(700+i, v[i]);
	}
	
	//the currentChoice is the default rule
	currentChoice = selection::getDefaultRules();
}
