/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "solutionwidget.h"
#include "tilewidget.h"
#include "operationwidget.h"
#include <QtGui>

//Widget definition of the solving module
SolutionWidget::SolutionWidget( QWidget *parent ) : QWidget( parent) {
	//vertical layout
	QVBoxLayout *vbox_Layout = new QVBoxLayout( this );
	
	vbox_Layout->setSpacing(0);
	vbox_Layout->setMargin(0);
	
	//pixmaps definitions for icons
	QPixmap *pixEqual = new QPixmap( QString(":/images/big_equal.png") );
	QPixmap *pixrubber = new QPixmap( QString(":/images/eraser.png") );
	QPixmap *pixReinit = new QPixmap( QString(":/images/reinit.png") );
	
	//operation tiles on a line
	QWidget *hbox_oper = new QWidget( this );
	QHBoxLayout *hbox_oper_Layout = new QHBoxLayout( hbox_oper );
	hbox_oper_Layout->setSpacing(0);
	hbox_oper_Layout->setMargin(0);
	
	//4 elementary operations
	hbox_oper_Layout->addStretch( 1 );
	int i;
	for(i=0; i<4; i++) {
		OperationWidget *op = new OperationWidget( this, 300+i, OperationWidget::Plus+i, true);
		connect( op, SIGNAL(value(int,int)), this, SLOT(tileClicked(int,int)) );
		hbox_oper_Layout->addWidget( op );
	}
	hbox_oper_Layout->addStretch( 1 );
	
	//buttons to clean the solving module
	QWidget *hbox_clean = new QWidget( this );
	QHBoxLayout *hbox_clean_Layout = new QHBoxLayout( hbox_clean );
	hbox_clean_Layout->setSpacing(0);
	hbox_clean_Layout->setMargin(0);
	
	QPushButton *pbrubber = new QPushButton( "", hbox_clean);
	pbrubber->setFlat( true );
	pbrubber->setFixedSize( 32, 32 );
	pbrubber->setIcon( *pixrubber );
	pbrubber->setIconSize( QSize(28,28) );
	pbrubber->setFocusPolicy(Qt::NoFocus);
	connect( pbrubber, SIGNAL(clicked()), this, SLOT(rubber()) );
	
	QPushButton *pbReinit = new QPushButton( "", hbox_clean);
	pbReinit->setFlat( true );
	pbReinit->setFixedSize( 32, 32 );
	pbReinit->setIcon( *pixReinit );
	pbReinit->setIconSize( QSize(28,28) );
	pbReinit->setFocusPolicy(Qt::NoFocus);
	connect( pbReinit, SIGNAL(clicked()), this, SLOT(init()) );
	
	hbox_clean_Layout->addStretch( 1 );
	hbox_clean_Layout->addWidget( pbrubber );
	hbox_clean_Layout->addWidget( pbReinit );
	hbox_clean_Layout->addStretch( 1 );
	
	//solutions grid
	QWidget *solutionGrid = new QWidget( this );
	QGridLayout *solutionGridLayout = new QGridLayout( solutionGrid );
	int j;
	
	int tileId=100; //unique id for each tile of the grid
	for(i=0; i<5; i++) {
		for(j=0; j<3; j++) {
			TileWidget *tile = new TileWidget(solutionGrid, tileId, 2);
			tileId++;
			solutionGridLayout->addWidget(tile , i, j*2, 1, 1, Qt::AlignTop);
			connect(this, SIGNAL(changetile(int, int)), tile, SLOT(displayRefresh(int, int)) );
			connect(this, SIGNAL(changeStatus(int, int)), tile, SLOT(updateStatus(int, int)) );
			connect(this, SIGNAL(clickNotPossible(int)), tile, SLOT(clickNotPossible(int)) );
			if(j==2) {
				connect( tile, SIGNAL(value(int,int)), this, SLOT(tileClicked(int,int)) );
			}
		}
		
		OperationWidget *operation = new OperationWidget( solutionGrid, 400+i, 0, false);
		connect(this, SIGNAL(changetile(int, int)), operation, SLOT(changeOperation(int, int)) );
		solutionGridLayout->addWidget(operation, i, 1);
		
		QLabel *labelEqual = new QLabel( solutionGrid );
		labelEqual->setPixmap( *pixEqual );
		labelEqual->setFixedSize( 32, 32 );
		labelEqual->setAlignment( Qt::AlignHCenter );
		solutionGridLayout->addWidget(labelEqual , i, 3);
	}
	
	//add the three mains components to the vertical layout
	vbox_Layout->addWidget(hbox_oper);
	vbox_Layout->addWidget(solutionGrid);
	vbox_Layout->addWidget(hbox_clean);
	
	//initialization
	position=100;
	x=1; y=1;
	total.assign(7,0);
	a.assign(7,0);
	b.assign(7,0);
	c.assign(7,0);
	
	//backward link between tiles
	previousTile.assign(115,-1);
	
	//solving module is locked by default
}

//function called when a given tile of value val has been clicked
//to distinguish tiles and operations, operations have value 501, 502, 503 and 504
void SolutionWidget::tileClicked(int numW, int val) {
	if(locked) return;
	if(numW>=300 && numW <=304) {
		//click on an operation tile
		if(y==2) {
			emit changetile(399+x, val);
			b[x]=val;
			y++;
		} else
		if(y==3) {
			//we're waiting a tile, so we modify the current line's operation
			emit changetile(399+x, val);
			b[x]=val;
		} else
		if(y==1 && x>1) {
			//convention : we automatically use previous line result
			a[x]=total[x-1];
			position--;
			emit changeStatus(position, 3);
			position++;
			emit changetile(position, total[x-1]);
			emit changeStatus(position, 3);
			previousTile[position]=position-1;
			position++;
			emit changetile(399+x, val);
			b[x]=val;
			y += 2;
		}
	} else  {
		//compute line's result and check that operation is valid
		//check is necessary before modifying the tile
		if(y==3) {
			c[x]=val;
			if(b[x]==OperationWidget::Plus) {
				total[x] = a[x] + c[x];
			} else
			if(b[x]==OperationWidget::Minus) {
				total[x] = a[x] - c[x];
				if(total[x]<=0) { //forbidden operation
					emit changeStatus(numW, 0);
					return;
				}
			} else
			if(b[x]==OperationWidget::Multiply) {
				total[x] = a[x] * c[x];
			} else
			if(b[x]==OperationWidget::Divide) {
				total[x] = a[x] / c[x];
				if(a[x]%c[x] != 0){ //forbidden operation
					emit changeStatus(numW, 0);
					return;
				}
			}
		}
		
		//modify current tile
		if(y==1 || y==3) {
			previousTile[position]=numW;
			//modify grid's current tile
			emit changetile(position, val);
			emit changeStatus(position, 3);
			position++;
			if(position==115) position=100;
			if(y==1) a[x]=val;
			//if(y==3) c=val; //note : already done above
		} else
		if(y==2) {
			//we're waiting an operation, so we modify the previous tile
			emit changeStatus(previousTile[position-1], 0);	//change color of previous tile
			previousTile[position-1]=numW;
			emit changetile(position-1, val);
			a[x]=val;
		}
		
		//increment y and change line's last tile
		if(y==1) {
			//if we are on the first line, check whether we have reached the target
			//this allows to solve problems with one tile only
			if(x==1 && (a[x] == targetRange.first || a[x] == targetRange.second) ) {
				//the tile at the end of the first line is of ID 102
				emit changetile(102, a[x]);
				emit changeStatus(102, 2);
				emit victory();
				return;
			}
			y++;
		} else
		if(y==3) {
			emit changetile(position, total[x]);
			emit changeStatus(position, 0);
			y=1; x++;
			position++;
		}
		
		//target was found
		if(total[x-1]==targetRange.first || total[x-1]==targetRange.second){
			changeStatus(position-1, 2);
			emit victory();
		}
	}
}

//clean all tiles
void SolutionWidget::init() {
	int i;
	
	//clean value tiles
	for(i=0; i<15; i++) {
		emit changetile(100+i, 0);
		emit changeStatus(100+i, 4);
		previousTile[100+i]=-1;
	}
	
	//clean operation tiles
	for(i=0; i<5; i++) {
		emit changetile(400+i, 0);
	}
	
	//clean problem tiles
	for(int i=0;i<6;i++){
		emit changeStatus(i,0);
	}
	
	//position state set back to its initial value
	position=100;
	x=1; y=1;
	total.assign(7,0);
	a.assign(7,0);
	b.assign(7,0);
	c.assign(7,0);
}

void SolutionWidget::rubber() {
	if(locked) return;
	if(y==1) {
		if(position==100) return; //nothing to do
		//delete previous line's result and previous tile
		position--;
		emit changetile(position, 0);
		emit changeStatus(position, 4);
		position--;
		emit changeStatus(previousTile[position], 0);
		previousTile[position]=-1;
		emit changetile(position, 0);
		emit changeStatus(position, 4);
		y=3;
		x--;
	} else
	if(y==2) {
		//we're waiting an operation tile, so we delete the previous value tile
		position--;
		y--;
		emit changeStatus(previousTile[position], 0);
		previousTile[position]=-1;
		emit changetile(position, 0);
		emit changeStatus(position, 4);
	} else
	if(y==3) {
		//delete operation tile
		emit changetile(399+x, 0);
		y--;
	}
}

//lock/unlock solving module
void SolutionWidget::lockModule(bool blocage){
	if(blocage){
		locked=true;
	} else {
		locked=false;
	}
	
	//lock white tiles
	for(int i=0; i<15; i++) {
		emit clickNotPossible(100+i);
	}
}
