/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tilewidget.h"
#include "selection.h"

#include <QtGui>

//create a widget with a Lcd display to represent a tile
TileWidget::TileWidget( QWidget *parent, int numW, int type_tileW)
        : QWidget( parent) {
	
	//widget index and type initialization
	//num is a unique id wich must be different for each tile
	num=numW;
	type_tile=type_tileW;
	
	QVBoxLayout *vboxLayout = new QVBoxLayout(this);
	vboxLayout->setSpacing(0);
	vboxLayout->setMargin(0);
	
	//lcd display
	lcd = new QLCDNumber(this);
	
	if (type_tile==0) {
		lcd->setNumDigits( 1 );
		lcd->setFixedSize( 32, 51 );
	} else {
		lcd->setNumDigits( 3 );
		lcd->setFixedHeight( 28 );
	}
	
	lcd->setSegmentStyle( QLCDNumber::Flat );
	lcd->setAutoFillBackground ( true );
	
	if (type_tile==0) {
		//target tile
		addColor(orangeColor); clickPossible=false; isVariableTile=true;
	} else if (type_tile==1){
		//selection tile
		addColor(whiteColor); clickPossible=true; isVariableTile=true;
	} else {
		//solving tile
		addColor(grayColor_total); clickPossible=false; isVariableTile=false;
	}
	
	vboxLayout->addWidget(lcd);
	
	//create timer
	timer= new QTimer;
	connect(timer, SIGNAL(timeout()), this, SLOT(pbClicked()) );
}

void TileWidget::pbClicked() {
	emit change(num, increaseTile);
}

void TileWidget::displayRefresh(int numW, int value) {
	//signal is used only if widget index is the same
	//a bit unsatisfying, but effective to avoid using widget's name
	if(num==numW) {
		if(numW>=100){	//case of a lcd in the solving widget
			if (value <=999) {lcd->setNumDigits(3);}
			else if(value <=9999) {lcd->setNumDigits(4);}
			else if(value <=99999) {lcd->setNumDigits(5);}
			else if(value <=999999) {lcd->setNumDigits(6);}
			else if(value <=9999999) {lcd->setNumDigits(7);}
			else if(value <=99999999) {lcd->setNumDigits(8);}
			else lcd->setNumDigits(9);
		}
		lcd->display(value);
		
		//display only used tiles in the board module
		if(numW < 10) {
			if(numW>=selection::nb_tiles){
				hide();
			}
			else{
				show();
			}
		}
		
		//display only used tiles in the solving module
		if(numW >= 100 && numW < 115) {
			int nbLine = selection::nb_tiles - 1;
			int currentLine = 1 + (numW - 100) / 3;
			
			if(currentLine > nbLine){
				hide();
			}
			else{
				show();
			}
		}
	}
}

void TileWidget::updateStatus(int numW, int value) {
	if(num==numW) {
		if(value==0){		//tile waiting a click
			clickPossible=true;
			addColor(whiteColor);
		} else
		if(value==1){		//unactive tile
			clickPossible=false;
			addColor(whiteColor);
		} else
		if(value==2){		//winning tile
			clickPossible=false;
			addColor(orangeColor);
		} else
		if(value==3){		//used tile
			clickPossible=false;
			addColor(grayColor);
		} else {		//hidden tile
			clickPossible=false;
			addColor(grayColor_total);
		}
	}
}

//forbid clicks
void TileWidget::clickNotPossible(int numW){
	if(num==numW) {
		clickPossible=false;
	}
}

//catch mouse wheel event (to adjust tile value)
void TileWidget::wheelEvent(QWheelEvent *event) {
	if (!isVariableTile) return;
	event->accept();
	if (event->delta()>0){
		increaseTile=true;
	} else {
		increaseTile=false;
	}
	pbClicked();
}

//emit tile value when clicked
void TileWidget::mousePressEvent(QMouseEvent *event) {
	if(isVariableTile){
		event->accept();
		if(event->button()==Qt::RightButton) {
			increaseTile=false;
		} else {
			increaseTile=true;
		}
		pbClicked();
		timer->start(150); //launch the timer to repeat the action
	} else
	if(clickPossible){
		event->accept();
		clickPossible=false;
		addColor(grayColor);
		emit value(num,lcd->intValue());
	}
}

//detect button release
void TileWidget::mouseReleaseEvent ( QMouseEvent *event) {
	event->accept();
	timer->stop();
}

//set/unset whether the tile is adjustable
void TileWidget::isVariable(bool isVariable_init) {
	isVariableTile = isVariable_init;
}

//set tile's color
void TileWidget::addColor(int tileColor) {
	if (tileColor==orangeColor) {
		lcd->setPalette( orangePalette );
	} else
	if (tileColor==whiteColor) {
		lcd->setPalette( whitePalette );
	} else
	if (tileColor==grayColor) {
		lcd->setPalette( grayPalette );
	} else {
		lcd->setPalette( grayPalette_total );
	}
}
