/***************************************************************************
 *   Copyright (C) 2006-2012 by Julien Lemoine, Simon Viennot              *
 *   Web site: http://kitsune.tuxfamily.org                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TILEWIDGET_H
#define TILEWIDGET_H

#include <QWidget>
#include <QWheelEvent>
#include <QMouseEvent>

#include "global.h"

class QLCDNumber;
class QTimer;

//Widget with a Lcd display to represent a tile
class TileWidget : public QWidget, kitsuneColors {
	Q_OBJECT
	
  public:
	TileWidget( QWidget *parent=0, int numW=0, int type_tile=0);
	
	//graphic element
	QLCDNumber *lcd;
	
	//widget index
	int num;
	
	//tile type : 0 for target tiles, 1 for selection tiles, 2 for solving tiles
	int type_tile;
	
	//true if the value is adjustable by the user
	bool isVariableTile;
	
	//false if the tile has already been used
	bool clickPossible;
	
	//timer for click repetition
	QTimer *timer;
	bool increaseTile;
	
	private slots :
	void pbClicked();

	public slots :
	void displayRefresh(int numW, int value);
	void updateStatus(int numW, int value);
	void isVariable(bool isVariableTile);    //set/unset whether the tile is adjustable
	void addColor(int tileColor);
	void clickNotPossible(int numW);
	
	void mousePressEvent(QMouseEvent *);
	void mouseReleaseEvent ( QMouseEvent *);
	void wheelEvent(QWheelEvent *event);
	
	signals :
	void change(int, bool); //connected directly to selection_change
	void value(int, int);  //emit the tiles's value
};

#endif
